//------------------------------------------------------------------------------
//
// File:    apsfile.cc
//
// Purpose: General purpose procedures for file processing
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "apsfile.h"
#include "newstr.h"
#include <limits.h>
#include <sys/file.h>
#include <unistd.h>

APSFile :: APSFile( const char * aFileName, const char * aMode )
{
  FileName = newStr( aFileName );
  Mode     = newStr( aMode );
  pHandler = NULL;
  opened   = 0;
}

APSFile :: ~APSFile( void )
{
  if( IfOpened() )
    Close();

  delete [] FileName;
  delete [] Mode;
}

FILE * APSFile :: GetHandlerPtr( void ) const
{
  return( pHandler );
}

int APSFile :: GetDescriptor( void ) const
{
  return( fileno( pHandler ) );
}

const char * APSFile :: GetFileName( void ) const
{
  return( FileName );
}

int APSFile :: IfOpened( void ) const
{
  return( opened );
}

int APSFile :: Open( void )
{
  if( !IfOpened() ) {
    pHandler = fopen( FileName, Mode );
    if( pHandler == NULL ) {
      return( 1 );
    }
  }
  else {
    return( 2 );
  }

  opened = 1;

  return( 0 );
}

int APSFile :: Close( void )
{
  if( IfOpened() ) {
    if( fclose( pHandler ) ) {
      return( 1 );
    }
  }
  else {
    return( 2 );
  }

  opened = 0;
  pHandler = NULL;

  return( 0 );
}

off_t APSFile :: LSeek( off_t Offset, int Whence ) const
{
  if( IfOpened() ) {
    return( lseek( GetDescriptor(), Offset, Whence ) );
  }

  return( -1 );
}

int APSFile :: IfExists( void ) const
{
  struct stat statBuffer;

  return( stat( GetFileName(), &statBuffer ) == 0 );
}

int APSFile :: GetDevice( dev_t * p_dev ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 1 );

  *p_dev = statBuffer.st_dev;

  return( 0 );
}

ino_t APSFile :: GetSerialNumber( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 0 );

  return( statBuffer.st_ino );
}

mode_t APSFile :: GetMode( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( INT_MAX );

  return( statBuffer.st_mode );
}

int APSFile :: GetLinkNumber( nlink_t * p_nlink ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 1 );

  *p_nlink = statBuffer.st_nlink;

  return( 0 );
}

int APSFile :: GetUserID( uid_t * p_uid ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 1 );

  *p_uid = statBuffer.st_uid;

  return( 0 );
}

int APSFile :: GetGroupID( gid_t * p_gid ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 1 );

  *p_gid = statBuffer.st_gid;

  return( 0 );
}

int APSFile :: GetDevID( dev_t * p_dev ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( 1 );

  *p_dev = statBuffer.st_rdev;

  return( 0 );
}

off_t  APSFile :: GetLength( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_size );
}

time_t APSFile :: GetLastAccess( void )	const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_atime );
}

time_t APSFile :: GetLastModify( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_mtime );
}

time_t APSFile :: GetLastStatus( void )	const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_ctime );
}

long APSFile :: GetBlockSize( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_blksize );
}

long APSFile :: GetBlockNumber( void ) const
{
  struct stat statBuffer;

  if( stat( GetFileName(), &statBuffer ) == -1 )
    return( -1 );

  return( statBuffer.st_blocks );
}

int APSFile :: Lock( void ) const
{
  //  return( flock( GetDescriptor(), LOCK_EX ) );
  return 0;
}

int APSFile :: UnLock( void ) const
{
  //  return( flock( GetDescriptor(), LOCK_UN ) ); 
  return 0;
}
