//------------------------------------------------------------------------------
//
// File:    apsreadcfg.cc
//
// Purpose: Class for reading config
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 06.03.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>

#include "apsreadcfg.h"
#include "apsconfig.h"
#include "apsistrfile.h"
#include "apsmodulecfgreader.h"

const int MAX_STRING_LENGTH = 1024;
const int MAX_MSG_LENGTH    = 1024;

//------------------------------- APSReadConfig ---------------------------------

APSReadConfig :: APSReadConfig( APSSubModule * pSubModule, const char * pConfigFilePath )
{
  pModule = new APSModuleConfigReader( pSubModule );
  pFile   = new APSIStrFile( pConfigFilePath );
}

APSReadConfig :: ~APSReadConfig( void )
{
  delete pFile;
  delete pModule;
}

int APSReadConfig :: Read( APSConfig * pConfig )
{
  char                 Str[ MAX_STRING_LENGTH ];
  char                 ParamType[ MAX_STRING_LENGTH ];
  char                 ParamName[ MAX_STRING_LENGTH ];
  char                 ParamValue[ MAX_STRING_LENGTH ];
  char                 NullParam[ MAX_STRING_LENGTH ];
  int                  IntParam;
  double               DoubleParam;
  char                 Msg[ MAX_MSG_LENGTH ];
  const APSParameter * pParameter;
  int                  RetCode;

  RetCode = 0;

  if( !pFile->Open() ) {
    while( !pFile->GetStr( Str, MAX_STRING_LENGTH ) ) {
      if( ( Str[ 0 ] != '#') && ( strcmp("\n", Str ) ) ) {
        if( sscanf( Str, "%s%s%s%s", ParamType, ParamName, ParamValue, NullParam ) == 3 ) {
          pParameter = pConfig->FindParameter( ParamName );

          if( pParameter ) {
            if( pConfig->IfExistsParameter( ParamName ) ) {
              sprintf( Msg, "File %s Parameter name `%s`\n", pFile->GetFileName(), ParamName );
              pModule->ErrorMessage( APS_MODULE_CFG_READER_EXISTS, Msg );
              RetCode = APS_MODULE_CFG_READER_EXISTS;
              break;
            }
          }
          else {
            sprintf( Msg, "File %s Parameter name `%s`\n", pFile->GetFileName(), ParamName );
            pModule->ErrorMessage( APS_MODULE_CFG_READER_NOT_FOUND, Msg );
            RetCode = APS_MODULE_CFG_READER_NOT_FOUND;
            break;
          }

          if( !strcmp( "int", ParamType ) ) {
            if( pParameter->GetType() == PARAM_INTEGER ) {
              if( sscanf( ParamValue, "%d", &IntParam ) == 1 ) {
                pConfig->AddIntegerParameter( ParamName, IntParam );
              }
              else {
                sprintf( Msg, "File %s Parameter name `%s` Value `%s` is not integer.\n", pFile->GetFileName(), ParamName, ParamValue );
                pModule->ErrorMessage( APS_MODULE_CFG_READER_VALUE, Msg );
                RetCode = APS_MODULE_CFG_READER_VALUE;
                break;
              }
            }
            else {
              sprintf( Msg, "File %s Parameter name `%s` is not integer.\n", pFile->GetFileName(), ParamName );
              pModule->ErrorMessage( APS_MODULE_CFG_READER_TYPE, Msg );
              RetCode = APS_MODULE_CFG_READER_TYPE;
              break;
            }
          }
          else {
            if( !strcmp( "double", ParamType ) ) {
              if( pParameter->GetType() == PARAM_DOUBLE ) {
                if( sscanf( ParamValue, "%lf", &DoubleParam ) == 1 ) {
                  pConfig->AddDoubleParameter( ParamName, DoubleParam );
                }
                else {
                  sprintf( Msg, "File %s Parameter name `%s` Value `%s` is not double.\n", pFile->GetFileName(), ParamName, ParamValue );
                  pModule->ErrorMessage( APS_MODULE_CFG_READER_VALUE, Msg );
                  RetCode = APS_MODULE_CFG_READER_VALUE;
                  break;
                }
              }
              else {
                sprintf( Msg, "File %s Parameter name `%s` is not double.\n", pFile->GetFileName(), ParamName );
                pModule->ErrorMessage( APS_MODULE_CFG_READER_TYPE, Msg );
                RetCode = APS_MODULE_CFG_READER_TYPE;
                break;
              }
            }
            else {
              if( !strcmp( "string", ParamType ) ) {
                if( pParameter->GetType() == PARAM_STRING ) {
                  pConfig->AddStringParameter( ParamName, ParamValue );
                }
                else {
                  sprintf( Msg, "File %s Parameter name `%s` is not string.\n", pFile->GetFileName(), ParamName );
                  pModule->ErrorMessage( APS_MODULE_CFG_READER_TYPE, Msg );
                  RetCode = APS_MODULE_CFG_READER_TYPE;
                  break;
                }
              }
              else {
                sprintf( Msg, "File %s Parameter type `%s`\n", pFile->GetFileName(), ParamType );
                pModule->ErrorMessage( APS_MODULE_CFG_READER_WRONG_TYPE, Msg );
                RetCode = APS_MODULE_CFG_READER_WRONG_TYPE;
                break;
              }
            }
          }
        }
        else {
          sprintf( Msg, "File %s String `%s`\n", pFile->GetFileName(), Str );
          pModule->ErrorMessage( APS_MODULE_CFG_READER_PARSE_LINE, Msg );
          RetCode = APS_MODULE_CFG_READER_PARSE_LINE;
          break;
        }
      }
    }

    pFile->Close();
  }
  else {
    sprintf( Msg, "File %s not found\n", pFile->GetFileName() );
    pModule->ErrorMessage( APS_MODULE_CFG_READER_FILE_NOT_FOUND, Msg );
    RetCode = APS_MODULE_CFG_READER_FILE_NOT_FOUND;
  }

  return( RetCode );
}
